/* -*- Mode: C; tab-width: 5; indent-tabs-mode: t; c-basic-offset: 5 -*- */
/* NetworkManager -- Network link manager

    SPDX-License-Identifier: GPL-2.0-or-later

    SPDX-FileCopyrightText: 2012 Guido Günther <agx@sigxcpu.org>
*/

#ifndef NM_IODINE_SERVICE_DEFINES_H
#define NM_IODINE_SERVICE_DEFINES_H

#define NM_DBUS_SERVICE_IODINE "org.freedesktop.NetworkManager.iodine"
#define NM_DBUS_INTERFACE_IODINE "org.freedesktop.NetworkManager.iodine"
#define NM_DBUS_PATH_IODINE "/org/freedesktop/NetworkManager/iodine"

#define NM_IODINE_KEY_TOPDOMAIN "topdomain"
#define NM_IODINE_KEY_NAMESERVER "nameserver"
#define NM_IODINE_KEY_FRAGSIZE "fragsize"
#define NM_IODINE_KEY_PASSWORD "password"

#endif /* NM_IODINE_SERVICE_DEFINES_H */
